
/*
 *  Psych0Day
 *  (K) 1997, 1998, 1999 by Andrew Ziem <ziem@iex.net>
 *  http://members.xoom.com/sdexter/psoft/
 *
 *     FIGURE.C -- figurin' routines
 *
 *
 *  Arguments used in most functions:
 *    daynumber = Gregorian day of year (1-366)
 *    year      = Gregorian year
 *
 *
 *  Warning:
 *    - not much checking is done to see if passed values are valid!
 *
 */


#include "figure.h"

/*
   ** Return: POEE day of year (1-366).  Simply subtract one if after the it's
   ** St. Tibb's Day.
   **
   ** Phhft: DAYNUM.C seems to already subtract one day; if it didnt, this
   ** would serve some purpose.
   **
 */

#define poeedayyear(a) a

#ifdef NOTUSED
unsigned int
  poeedayyear(unsigned int daynumber)
{
    if (isleap(year))
	return (daynumber > 60 ? daynumber - 1 : daynumber);	/* ? */
    else
	return (daynumber);

}
#endif


/*
   ** Return: POEE day of week (0-5, 0 = Sweetmorn)
   **
 */

unsigned int
  poeedayweek(unsigned int daynumber)
{
    unsigned int joe;

    joe = poeedayyear(daynumber) % 5L;
    return (joe ? joe : 5);

}


/*
   ** Return: POEE day of season (1-73).
   **
 */

unsigned int
  poeedayseason(unsigned int daynumber, YEAR year)
{
    unsigned int joe, sue;

    joe = poeedayyear(daynumber);
    if (daynumber > 60 && isleap(year))
	sue = (joe - 1) % 73L;
    else
	sue = joe % 73L;

    if (0 == sue)
	return 73;
    else
	return sue;

}


/*
   ** Return: POEE calendar season (0-4, 0=Chaos).
   **
 */

unsigned int
  poeeseason(unsigned int daynumber, YEAR year)
{

    return ((poeedayyear(daynumber) / 73) - (poeedayseason(daynumber, year) == 73 ? 1 : 0));

}


/*
   ** Return: year's Golden Number (1-19).
   ** Warning: untested.
   **
 */

#define golden_number(year)      ((year % 19) + 1)


/*
   ** Return: year's Epact (1-30), Gregorian method
   ** Accepts: positive years
   ** Warning: untested
   **
   ** "In the Gregorian calendar, the Epact is the age of the moon at the
   ** start of the year."
   **
   ** Adapted to C from the Calendar FAQ.
   **
 */

int
  epact(YEAR year)
{
    unsigned int epact = 0, century;

    century = (year / 100) + 1;

    epact = (11 * (golden_number(year) - 1)) % 30;
    epact = epact - (3 * century) / 4;
    epact = epact + (8 * century + 5) / 25;
    epact = epact + 8;
/* 5) Add or subtract 30 until the Epact lies between 1 and 30. */
    if (epact > 30)
	epact -= 30;

    return epact;

}				/* epact() */


/*
   ** Return: phase of the moon for a certian point in time (0-7, 0=new).
   **
   ** Stolen from Doorskel, where it was stolel from Lynn Nash, who hacked it
   ** from BSD Unix (I think).  Tweeked by myself.
   **
 */

unsigned int
  Moon_Phase(YEAR year, int diy)
{
    int phase;			/* Moon phase                  */


    /* Calculate the phase, using the magic numbers defined above. Note that
     * (phase and 7) is equivalent to (phase mod 8) and is needed on two days
     * per year (when the algorithm yields 8). */

    phase = (((((diy + epact(year)) * 6) + 11) % 177) / 22) & 7;

    if (8 == phase)
    {
	printf("debug: phase 8!!!\n");
	phase = (((((diy + epact(year)) * 6) + 11) % 177) / 22) % 8;
    }

    return phase;
}


/*
   ** Return: is date valid in the Gregorian calendar?  (0 = false, 1 = true)
   ** From: DAYNUM.C by Bob Stout.
   **
   ** Modified to not accept the year '0' and slightly optimized.
   **
 */

int
  checkdate(unsigned yr, unsigned mo, unsigned day)
{
    unsigned int days[] =
    {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};

    if ((1 > mo || 12 < mo) ||
	(1 > day || day > (days[mo - 1] + (2 == mo && isleap(yr)))) ||
	(0 == yr))
	return FALSE;
    else
	return TRUE;

}


/*
 * ltostr() by Jerry Coffin
 * modified by Psych0 use only base 26 (A through Z)
 */

char *
  lstr(long num, char *string, size_t max_chars)
{
    char remainder;
    int sign = 0;		/* number of digits occupied by the sign */


    if (num < 0)
    {
	sign = 1;
	num = -num;
    }

    --max_chars;
    string[max_chars] = NUL;

    for (max_chars--; max_chars > sign && num != 0; max_chars--)
    {
	remainder = (char) (num % BASE_LTOSTR);
	string[max_chars] = remainder + 'A' - 1;
	num /= BASE_LTOSTR;
    }

    if (sign)
	string[--max_chars] = '-';

    if (max_chars > 0)
	pmemset(string, ' ', max_chars + 1);

    return (char *) string + max_chars;
}				/* lstr() */



/*
   ** Return: Jusanotoron date
   ** By: Psych0
   **
   ** Hey: years may be off 'cause of that damn no year zero rule...
   **
 */

void
  jusanotoron_date(YEAR gyear, unsigned int gyday, unsigned int *jyear, unsigned int *jyday)
{
    /* our cute little epoch */
    *jyear = gyear + 585 + 1;	/* or minus one, or nothing... argh! */

    /* year begins May 28th each and every year */
    if (gyday >= 148 && gyday <= 366)
    {
	*jyday = gyday - 147;
    }
    else
    {
	*jyear -= 1;
	if (isleap(gyear))
	    *jyday = 366 - gyday;
	else
	    *jyday = 365 - gyday;
    }

    if (jyear >= 0)
	*jyear -= 1;

}				/* jusanotoron_date() */


/* end of file "FIGURE.C" */
