/*
 *  Psych0Day
 *  (K) 1997, 1998, 1999 by Andrew Ziem <ziem@iex.net>
 *  http://members.xoom.com/sdexter/psoft/
 *
 *     PDAY.C -- main source
 *
 */

#define INC_DATABASE
#define INC_CALENDAR
#define NO_JEWISH		/* buggy */

#include "pday.h"

#include "data.h"
#include "figure.h"
#if defined(INC_DATABASE)
#include "library.h"
#endif
#if defined(INC_CALENDAR)
#include "calendar.h"
#endif

void
  criterr(char const *errmsg,...)
{

    va_list args;

    va_start(args, errmsg);
    fprintf(stderr, "pday: ");
    vfprintf(stderr, errmsg, args);
    fputc('\n', stderr);
    va_end(args);

    exit(EXIT_FAILURE);
}				/* criterr() */


#if defined(INC_DATABASE)
void exitfunc(void)
{

    if (events && LLSnodePtr2First(ListEvents))
    {
	DATAENTRY dataentry;

	do
	{
	    LLSnodeDataTo(ListEvents, &dataentry);
	    FREE(dataentry.text);
	}
	while (LLSnodePtr2Next(ListEvents));

	events = 0;
    }

}				/* exitfunc() */
#endif


void
  usage(void)
{

#define USAGE  \
 \
   "\nUsage:\n" \
   "PDAY [{-|+}C[x]] [+d] [-Fdd[mm[yyyy]]] [-I] [{-|+}L] [-LIB=drive:\\dir\\]\n\n" \
   "    c[x]:            toggle display of calendars\n" \
   "    d:               debug\n" \
   "    f[mm[yyyy]]:     force date\n" \
   "    i:               information\n" \
   "    l[x]:            toggle library support\n" \
   "    lib=x:           library directory\n" \
   "\n" \
   "For further information, see the enclosed documentation.\n"

    printf(USAGE);
    exit(EXIT_SUCCESS);

}




int
  main(int argc, char *argv[])
{
    time_t t;			/* long same as time_t */
    struct tm *eris;

    int i;


    /* say hello */
    printf("\n" PROG_NAME " version " PROG_VER " -- " PROG_DATE "\n");
    printf("(K) 1997, 1998, 1999 by Andrew Ziem.  All Rites Reversed.\n");

#if defined(INC_DATABASE)
    /* initilize safeties */
    atexit(exitfunc);

    /* initilize linked list */
    LLSsystemInit(2);
    ListEvents = LLScreate(SIZEOF_DATAENTRY);

    if (ERR_MEMORY == ListEvents)
	criterr("unable to initilize primary linked list (memory error)\n");
#endif

    tzset();
    t = time(NULL);
    eris = localtime(&t);
    greg.year = eris->tm_year + 1900;	/* calendar year */
    greg.month = eris->tm_mon + 1;	/* month of year (1..12) */
    greg.mday = eris->tm_mday;	/* day of month (1..31) */

    /* initilize command line */
    for (i = 1; i < argc; i++)
    {
	char opt[100], ss[10];


	strcpy(opt, argv[i]);

	if (opt[0] == '/' || opt[0] == '-' || opt[0] == '+')
	{
	    switch (toupper(opt[1]))
	    {

		case 'C':

		    /* calendars */
		    switch (toupper(opt[2]))
		    {
			case 'D':

			    opts.showdiscord = (opt[0] == '+');
			    break;

			case 'E':

			    opts.showgue = (opt[0] == '+');
			    break;

			case 'F':

			    opts.showfrench = (opt[0] == '+');
			    break;

			case 'G':

			    opts.showgregorian = (opt[0] == '+');
			    break;

			case 'I':

			    opts.showilluminati = (opt[0] == '+');
			    break;

			case 'J':

			    opts.showjulian = (opt[0] == '+');
			    break;

			case 'L':

			    opts.showgoddesslunar = (opt[0] == '+');
			    break;

			case 'M':

			    opts.showmoon = (opt[0] == '+');
			    break;

			case 'T':

			    opts.showthelemic = (opt[0] == '+');
			    break;

			case 'U':

			    opts.showjusanotoron = (opt[0] == '+');
			    break;

			default:

			    opts.showdiscord = opts.showfrench = opts.showgregorian =
				opts.showilluminati = opts.showgue = opts.showjulian =
				opts.showgoddesslunar = opts.showmoon =
				opts.showthelemic = opts.showjusanotoron = (opt[0] == '+');
			    break;

		    }		/* 'C' (calendars */
		    break;

		case 'D':

		    mopts.debug = (opt[0] == '+');
		    break;

		case 'L':

#if defined(INC_DATABASE)
		    /* 0123456 */
		    /* -lib=dirname */

		    opt[4] = NUL;

		    if (0 == stricmp(&opt[1], "lib"))
		    {
			add_library(&opt[5]);
		    }
		    else
			switch (toupper(opt[2]))
			{
			    case 'E':

				opts.showevents = (opt[0] == '+');
				break;

			    case 'B':

				opts.showbirthdays = (opt[0] == '+');
				break;

			    case 'R':

				opts.showreminders = (opt[0] == '+');
				break;

			    default:

				opts.showevents = opts.showbirthdays =
				    opts.showreminders = (opt[0] == '+');
			}
#else
		    printf("Sorry, library functions not linked.\n");
#endif
		    break;


		case 'F':

		    strncpy(ss, opt + 2, 2);
		    ss[2] = NUL;
		    greg.mday = atoi(ss);

		    if (strlen(opt + 2) >= 4)
		    {
			strncpy(ss, opt + 4, 2);
			ss[2] = NUL;
			greg.month = atoi(strncpy(ss, opt + 4, 2));
		    }

		    if (strlen(opt + 2) >= 7)
		    {
			strncpy(ss, opt + 6, 7);
			ss[7] = NUL;
			greg.year = atoi(ss);
		    }

		    break;

		case 'I':

#if defined(__EMX__) && !defined(__RSXNT__)
		    printf("\nCompiled on " __DATE__ " at " __TIME__ " using " COMPILER_NAME " (runtime revision %X).\n", _emx_rev);
#elif defined(__RSXNT__)
		    printf("\nCompiled on " __DATE__ " at " __TIME__ " using " COMPILER_NAME " (runtime version %X.%02X).\n", (_emx_rev >> 16) >> 8, (_emx_rev & 0xFFFF) & 0xFF);
#else
		    printf("\nCompiled on " __DATE__ " at " __TIME__ " using " COMPILER_NAME ".\n");
#endif
		    printf("Environment variable 'TZ' set to '%s'.\n", getenv("TZ"));

#if defined(SUPERY2K)
		    printf("Year 2000+ overkill enabled.\n");
#endif
		    printf(CONTACT);

		    printf("\nPortions copyright 1993-1995, Scott E. Lee.\n");
		    printf("For further information, see the enclosed documentation.\n");

		    exit(EXIT_SUCCESS);

		case 'S':

		    opts.sort = (opt[0] == '+');
		    break;

		default:

//           printf("debug: argv[%i] = %s\n", i, argv[i]);
		    usage();
	    }			/* switch */
	}			/* if... */
	else
	    usage();
    }				/* for */


    if (!checkdate(greg.year, greg.month, greg.mday))
	criterr("\ninvalid date:\n day=%d\n month=%d\n year=%d", greg.mday, greg.month, greg.year);

    greg.yday = daynum(greg.year, greg.month, greg.mday);
    greg.wday = dow(greg.year, greg.month, greg.mday);
    greg.jdn = ymd_to_jdnl(greg.year, greg.month, greg.mday, 1);

/* we need other things before we can pretend to be this smart:

   opts.showgregorian = (greg.JDN >= LASTJULJDN);
   opts.showjulian = !opts.showgregorian;
 */

#if defined(INC_CALENDAR)
    show_days();
#endif

#if defined(INC_DATABASE)
    if (opts.showevents || opts.showbirthdays)
    {
	if (0 == events)
	{
#if !defined(__RSXNT__)
	    add_library(".");
#endif
	}

#ifdef DEBUG_386
	printf("debug: show1\n");
#endif

	/* show */
	if ((opts.showbirthdays || opts.showevents) && events == 0)
	    printf("\nNo birthdays or other events found for today.\n");
	else if (opts.showbirthdays || opts.showevents)
	{
	    /* sort */
	    if (opts.sort)
	    {
		if (mopts.debug)
		{
		    printf("debug: sorting\n");
		}
		sort_data();
	    }

#ifdef DEBUG_386
//       printf("debug: show2\n");
#endif

	    /* show */
	    if (opts.showbirthdays)
	    {
		show_data(T_BIRTHDAY);
	    }
	    if (opts.showevents)
	    {
		show_data(T_EVENT);
	    }
	    if (mopts.debug)
	    {
		printf("\ndebug: %i total events\n", events);
	    }
	}
    }

#endif /* INC_DATABASE */

    printf("\nThat's all for today.\n");


    return 0;

}				/* main() */


/* end of file "PDAY.C" */
